@@ -9,8 +9,8 @@ loss.function = function(fitted, data, loss, extra.args, debug = FALSE) {
   else if (loss %in% c("pred", "pred-exact", "pred-lw", "pred-lw-cg")) {
 
     result = classification.error(node = extra.args$target, fitted = fitted,
-               prior = extra.args$prior, n = extra.args$n,
-               from = extra.args$from, data = data, loss = loss, debug = debug)
+               prior = extra.args$prior, extra.args = extra.args,
+               data = data, loss = loss, debug = debug)
 
   }#THEN
   else if (loss %in% c("cor", "cor-lw", "cor-lw-cg")) {
@@ -60,8 +60,9 @@ kfold.loss.postprocess = function(kcv, kcv.length, loss, extra.args, data) {
   else {
 
     # compute the mean of the observed values of the loss function, weighted
-    # to account for unequal-length splits.
-    mean = weighted.mean(unlist(sapply(kcv, '[', 'loss')), kcv.length)
+    # to account for unequal-length splits and for missing values within splits.
+    mean = weighted.mean(x = unlist(sapply(kcv, '[', 'loss')),
+                         w = unlist(sapply(kcv, '[', 'effective.size')))
 
   }#ELSE
 
@@ -74,13 +75,17 @@ mse.loss = function(node, fitted, data, loss, extra.args,
     debug = FALSE) {
 
   if (loss == "mse")
-    pred = gaussian.prediction(node, fitted, data)
+    pred = predict.backend(fitted = fitted, node = node, data = data,
+             method = "parents")
   else if (loss %in% c("mse-lw", "mse-lw-cg"))
-    pred = map.prediction(node, fitted, data, n = extra.args$n,
-             from = extra.args$from)
+    pred = predict.backend(fitted = fitted, node = node, data = data,
+             method = "bayes-lw", extra.args = extra.args)
+
+  effective.size = how.many(!is.na(pred) & !is.na(data[, node]))
 
   return(list(loss = mean((data[, node] - pred)^2, na.rm = TRUE),
-              predicted = pred, observed = data[, node]))
+              predicted = pred, observed = data[, node],
+              effective.size = effective.size))
 
 }#MSE.LOSS
 
@@ -89,41 +94,45 @@ predictive.correlation = function(node, fitted, n, from, data, loss,
     debug = FALSE) {
 
   if (loss == "cor")
-    pred = gaussian.prediction(node, fitted, data)
+    pred = predict.backend(fitted = fitted, node = node, data = data,
+             method = "parents")
   else if (loss %in% c("cor-lw", "cor-lw-cg"))
-    pred = map.prediction(node, fitted, data, n = n, from = from)
+    pred = predict.backend(fitted = fitted, node = node, data = data,
+             method = "bayes-lw", extra.args = list(n = n, from = from))
+
+  effective.size = how.many(!is.na(pred) & !is.na(data[, node]))
 
   if (((loss == "cor") && length(fitted[[node]]$parents) == 0))
     return(list(loss = NA, predicted = pred, observed = data[, node]))
   else
     return(list(loss = cor(data[, node], pred, use = "complete.obs"),
-                predicted = pred, observed = data[, node]))
+                predicted = pred, observed = data[, node],
+                effective.size = effective.size))
 
 }#PREDICTIVE.CORRELATION
 
 # log-likelihood loss function for gaussian and discrete networks.
-entropy.loss = function(fitted, data, keep = names(fitted), by.sample = FALSE,
-    debug = FALSE) {
+entropy.loss = function(fitted, data, debug = FALSE) {
 
-  list(loss = .Call(call_entropy_loss,
-                    fitted = fitted,
-                    data = data,
-                    by.sample = by.sample,
-                    keep = keep,
-                    debug = debug))
+  .Call(call_entropy_loss,
+        fitted = fitted,
+        data = data,
+        debug = debug)
 
 }#ENTROPY.LOSS
 
 # classification error as a loss function.
-classification.error = function(node, fitted, prior = NULL, n, from, data,
-    loss, debug = FALSE) {
+classification.error = function(node, fitted, prior = NULL, data, loss,
+    extra.args, debug = FALSE) {
 
   if (loss == "pred-exact")
     pred = naive.classifier(node, fitted, prior, data)
   else if (loss == "pred")
-    pred = discrete.prediction(node, fitted, data)
+    pred = predict.backend(fitted = fitted, node = node, data = data,
+             method = "parents")
   else if (loss %in% c("pred-lw", "pred-lw-cg"))
-    pred = map.prediction(node, fitted, data, n = n, from = from)
+    pred = predict.backend(fitted = fitted, node = node, data = data,
+             method = "bayes-lw", extra.args = extra.args)
 
   l = .Call(call_class_err,
             reference = .data.frame.column(data, node),
@@ -132,7 +141,10 @@ classification.error = function(node, fitted, prior = NULL, n, from, data,
   if (debug)
     cat("  > classification error for node", node, "is", l, ".\n")
 
-  return(list(loss = l, predicted = pred, observed = data[, node]))
+  effective.size = how.many(!is.na(pred) & !is.na(data[, node]))
+
+  return(list(loss = l, predicted = pred, observed = data[, node],
+              effective.size = effective.size))
 
 }#CLASSIFICATION.ERROR
 
